<?php
namespace Teknisa\COZ\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Coz01000ConsumoPlanejado extends \Zeedhi\Framework\Controller\Report {

	protected $crudService;
	protected $util;
	protected $dataSourceManager;
	protected $novoCodigo;
	protected $entityManager;
	protected $birtStrategy;

	public function __construct(ReportService $reportService, $crudService, $util, $novoCodigo, $entityManager, $birtStrategy) {
		$this->crudService    	= $crudService;
		$this->util             = $util;
		$this->novoCodigo    	= $novoCodigo;
		$this->entityManager    = $entityManager;
		$this->birtStrategy     = $birtStrategy;
		
		parent::__construct($reportService);
    }
	
	protected $reportMapping = array (
      'cus01000' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
                'DTINICIAL'     => 'P_DTINI',
                'DTFINAL'       => 'P_DTFIN',
                'CDCENTRAL'     => 'P_CDCENTRAL',
                'NRLISTA'       => 'P_NRLISTA',
                'NRCONSPLAN'    => 'P_NRCONSPLAN',
                'ORDENA'        => 'P_ORDENA',
                'CDPICTPROD'    => 'P_CDPICTPROD',
                'CDARVPRODINI'  => 'P_CDARVPRODINI',
                'CDARVPRODFIN'  => 'P_CDARVPRODFIN',
                'NRORG'         => 'P_NRORG'
            )
        ),
        'cus01001' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
                'DTINICIAL'     => 'P_DTINI',
                'DTFINAL'       => 'P_DTFIN',
                'CDCENTRAL'     => 'P_CDCENTRAL',
                'NRLISTA'       => 'P_NRLISTA',
                'NRCONSPLAN'    => 'P_NRCONSPLAN',
                'ORDENA'        => 'P_ORDENA',
                'CDPICTPROD'    => 'P_CDPICTPROD',
                'CDARVPRODINI'  => 'P_CDARVPRODINI',
                'CDARVPRODFIN'  => 'P_CDARVPRODFIN',
                'NRORG'         => 'P_NRORG'
            )
        ),
        'cus01002' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
                'DTINICIAL'     => 'P_DTINI',
                'DTFINAL'       => 'P_DTFIN',
                'CDCENTRAL'     => 'P_CDCENTRAL',
                'NRLISTA'       => 'P_NRLISTA',
                'NRCONSPLAN'    => 'P_NRCONSPLAN',
                'ORDENA'        => 'P_ORDENA',
                'CDPICTPROD'    => 'P_CDPICTPROD',
                'CDARVPRODINI'  => 'P_CDARVPRODINI',
                'CDARVPRODFIN'  => 'P_CDARVPRODFIN',
                'NRORG'         => 'P_NRORG'
            )
        )
    );
	
	public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response){
	    try{
	        
	        $row = $request->getRow();

    	    if(empty($row['NMSERVFILI'])) $row['NMSERVFILI'] = $this->filtraServicos($row);
	    	
    	    $row['NRLISTA'] = $this->gerarListaServ($row['NMSERVFILI'], $row['NRORG']);
    
    	    $nrListaConsplan = $this->gerarLista($row['NMSERVFILI'], $row['NRORG']);

    	    $cdprodini = empty($row['CDARVPRODINI']) ? '0' : $row['CDARVPRODINI'];
            if(empty($row['CDARVPRODFIN'])) $cdprodfin = 'Z';
            else $cdprodfin = str_pad($row['CDARVPRODFIN'], 10, "Z", STR_PAD_RIGHT);
            
            $row['CDARVPRODINI'] = $cdprodini;
            $row['CDARVPRODFIN'] = $cdprodfin;
            
    	    $row['NRCONSPLAN'] = $this->gerarNrConsplan($nrListaConsplan, $row);
    	    $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
	    }catch(\Exception $e){
	        $response->setError(new Error($e->getMessage(), 500));
	    }
	}
	
	public function filtraServicos($row){
		 $conditions = [
			 	array(
                    "columnName" => "NRORG",
                    "operator"   => '=',
                    "value"      =>  $this->crudService->getEnvironment()->getNrOrg()
                ),
                array(
                    "columnName" => 'CDCENTRAL',
                    "operator"   => '=',
                    "value"      => $row['CDCENTRAL']
                ),
                array(
                    "columnName" => 'CDGRUPFILI',
                    "operator"   => '=',
                    "value"      => $row['CDGRUPFILI']  === '' ? 'T' : $row['CDGRUPFILI']
                ),
                array(
                    "columnName" => 'CDGRUPSERV',
                    "operator"   => '=',
                    "value"      => $row['NMGRUPSERV'] === '' || sizeof($row['NMGRUPSERV']) == 0 ? 'T' : join('_',$row['NMGRUPSERV'])   
                ),
                array(
                    "columnName" => 'CDFILIAGREGA',
                    "operator"   => '=',
                    "value"      => $row['NMFILIAL'] === '' || sizeof($row['NMFILIAL']) == 0 ? 'T' : join('_',$row['NMFILIAL'])
                ),
                
            ];
         $dataset = $this->crudService->getFilteredDataSet("servico_por_filiais", $conditions);       
 		 $servicos = $dataset->getRows();
 		 
 		 return $servicos;
	}
	
	public function gerarListaServ($servicos, $nrorg) {
     try {   
            
        	$nrlista = $this->novoCodigo->getCodeByTableName('LISTA_SERVICO', $nrorg);
        	$sql = "INSERT INTO LISTA_SERVICO (NRLISTA, CDFILIAL, CDSERVFILI, NMSERVICO, CDGRUPSERV)
        	                    VALUES (:NRLISTA, :CDFILIAL, :CDSERVFILI, :NMSERVICO, :CDGRUPSERV)";
            
        	foreach ($servicos as $servico) {
                $stmt = $this->entityManager->getConnection()->prepare($sql);
                $stmt->execute(
                    array(
                        "NRLISTA"    => $nrlista,
                        "CDFILIAL"   => $servico['CDFILIAL'],
                        "CDSERVFILI" => $servico['CDSERVFILI'],
                        "NMSERVICO"  => $servico['NMSERVICO'],
                        "CDGRUPSERV" => empty($servico['CDGRUPSERV']) ? null : $servico['CDGRUPSERV']
                    )
                );
            }
            return $nrlista;
        } catch (\Exception $e) {
            throw $e;
        }
    }
	
	public function gerarLista($servicos, $nrorg) {
     try {  
        	$nrlista = $this->novoCodigo->getCodeByTableName('LISTA', $nrorg);
        	$sql = "INSERT INTO LISTA (NRLISTA, CDLISTA)
        	                    VALUES (:NRLISTA, :CDLISTA)";
            
        	foreach ($servicos as $servico) {
                $stmt = $this->entityManager->getConnection()->prepare($sql);
                $stmt->execute(
                    array(
                        "NRLISTA" => $nrlista,
                        "CDLISTA" => $servico['CDFILIAL'].$servico['CDSERVFILI']
                    )
                );
            }
            return $nrlista;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
	public function gerarNrConsplan($nrlista, $row) {
		try {
		    $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CONSPLAN_GERAL');
            $procedure->addParam(new Param('NRLISTA', Param::PARAM_INPUT, $nrlista, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('DTINICIAL', Param::PARAM_INPUT, $row['DTINICIAL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('DTFINAL', Param::PARAM_INPUT, $row['DTFINAL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('IDPRODUTOS', Param::PARAM_INPUT, 'CFP', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('IDSEPPRATO', Param::PARAM_INPUT, 'P', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('PEACRESCIMO', Param::PARAM_INPUT, 0, Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('IDARREDOND', Param::PARAM_INPUT, 'N', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('CDPRODUTOINI', Param::PARAM_INPUT, '0', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('CDPRODUTOFIN', Param::PARAM_INPUT, 'Z', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('CDOPERADOR', Param::PARAM_INPUT, $this->crudService->getEnvironment()->getCdOperador(), Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('NRCONSPLANTH', Param::PARAM_INPUT, 'GERANOVCOD', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('NRCONSPLAN', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 10));
            $nrconsplan = $procedure->execute()['NRCONSPLAN'];
            return $nrconsplan;
		} catch (\Exception $e) {
			throw $e;
    	}
	}
	
	public function excluirParametros(DTO\Request\Row $request, DTO\Response $response){
	    $row = $request->getRow();
	   if(isset($row["NRLISTA"])){
	        $this->util->deleteLista($row["NRLISTA"]);
    	    $connection = $this->entityManager->getConnection();
            $query="DELETE FROM CONSPLAN WHERE NRORG = :NRORG AND NRCONSPLAN = :NRCONSPLAN";
            $connection->executeQuery($query, array("NRORG" => $row['NRORG'], "NRCONSPLAN" => $row['NRCONSPLAN']));
	   }
	   
	}
	
	public function relatorioCustomizavel(DTO\Request\Row $request, DTO\Response $response) {
	    $row = $request->getRow();
	    
	    if(empty($row['NMSERVFILI'])) $row['NMSERVFILI'] = $this->filtraServicos($row);
	    $row['NRLISTA'] = $this->gerarListaServ($row['NMSERVFILI'], $row['NRORG']);
    	$nrListaConsplan = $this->gerarLista($row['NMSERVFILI'], $row['NRORG']);
    	$row['NRCONSPLAN'] = $this->gerarNrConsplan($nrListaConsplan, $row);
    	
    	$cdprodini = empty($row['CDARVPRODINI']) ? '0' : $row['CDARVPRODINI'];
        if(empty($row['CDARVPRODFIN'])) $cdprodfin = 'Z';
        else $cdprodfin = str_pad($row['CDARVPRODFIN'], 10, "Z", STR_PAD_RIGHT);
        
        $row['CDARVPRODINI'] = $cdprodini;
        $row['CDARVPRODFIN'] = $cdprodfin;
	    
	    $sql = "SELECT :P_CDCENTRAL AS CDCENTRAL, :P_NMCENTRAL AS NMCENTRAL, :P_DTINI AS DTINICIAL, :P_DTFIN AS DTFINAL, C.CDGRUPSERV AS CODIGOGRUPO, S.NMGRUPSERV AS GRUPOSERVICO, TO_CHAR(C.DT,'DD/MM/YYYY') AS DATA, TO_CHAR(C.DT,'Day') AS DIA, C.CDPRODPRODUCAO, PROD.NMPRODUTO AS PRODUTOACABADO, C.QTCOMENSAIS AS COMENSAIS, P.CDARVPROD AS CODIGO, DECODE(C.IDTPREQPROD, 'D', '* ','  ') || P.NMPRODUTO AS PRODUTO, C.QTPROD_PLA AS QTDE, P.NRORG, P.SGUNIDADE AS UN, C.IDTPREQPROD FROM ( SELECT MP.DT, MP.CDPRODPRODUCAO, MP.CDPRODUTO, SUM(MP.QTPROD_PLA) QTPROD_PLA, MP.QTCOMENSAIS, MP.IDTPREQPROD, MP.CDGRUPSERV FROM ( SELECT CONS.DTCONSPLAN AS DT, CONS.CDPRODUTO, SUM(CONS.QTCOMESTCPLA) AS QTCOMENSAIS, CONS.CDPRODPRODUCAO, SUM(CONS.QTCONSPLAN) QTPROD_PLA, CONS.IDTPREQPROD, CONS.CDGRUPSERV FROM ( SELECT DISTINCT C.CDPRATCONSPL, C.CDFAPRATCPLA, C.CDFILICPLA, C.CDSERVCPLA, C.DTCONSPLAN, MAX(C.QTCOMESTCPLA) AS QTCOMESTCPLA, SUM(C.QTCONSPLAN) AS QTCONSPLAN, R.CDPRODPRODUCAO, NVL(R.CDPRODUTO, C.CDPRODCPLA) AS  CDPRODUTO, R.IDTPREQPROD, SUM(NVL(C.QTCOMPLPRODCPLA,0) ) AS COMPLEMENTO, MIN(NVL(QTAMOSTRA,0) ) AS AMOSTRA, R.IDFINALIZACAO, S.CDGRUPSERV FROM CONSPLAN C LEFT JOIN TABLE ( RECEITA_UTIL(C.CDFILICPLA,C.CDSERVCPLA,C.CDPRATCONSPL,C.CDFAPRATCPLA,'0','Z',NULL,0,0,'S') ) R ON R.CDPRODPRODUCAO IS NOT NULL AND R.IDTPREQPROD <> 'E' INNER JOIN LISTA_SERVICO S ON S.NRLISTA = :P_NRLISTA AND S.CDFILIAL = C.CDFILICPLA AND S.CDSERVFILI = C.CDSERVCPLA WHERE C.NRCONSPLAN =:P_NRCONSPLAN AND ( E_AGREGADA(:P_CDCENTRAL,C.CDFILICPLA,C.CDSERVCPLA,C.CDTPPRATCPLA,C.DTCONSPLAN) > 0 ) AND C.IDTPREQPROD <> 'E' AND R.CDPRODUTO = C.CDPRODCPLA GROUP BY C.CDPRATCONSPL, C.CDFAPRATCPLA, C.CDFILICPLA, C.CDSERVCPLA, C.DTCONSPLAN, C.QTCOMESTCPLA, R.CDPRODPRODUCAO, NVL(R.CDPRODUTO, C.CDPRODCPLA), R.IDTPREQPROD, R.IDFINALIZACAO, S.CDGRUPSERV UNION SELECT DISTINCT C.CDPRATCONSPL, C.CDFAPRATCPLA, C.CDFILICPLA, C.CDSERVCPLA, C.DTCONSPLAN, MAX(C.QTCOMESTCPLA) AS QTCOMESTCPLA, SUM(NVL(C.QTCOMPLPRODCPLA,0) ) * MAX(C.QTCOMESTCPLA) AS QTCONSPLAN, R.CDPRODPRODUCAO, NVL(R.CDPRODUTO, C.CDPRODCPLA) AS  CDPRODUTO, 'D' AS IDTPREQPROD, 0 AS COMPLEMENTO, 0 AS AMOSTRA, R.IDFINALIZACAO, S.CDGRUPSERV FROM CONSPLAN C LEFT JOIN TABLE ( RECEITA_UTIL(C.CDFILICPLA,C.CDSERVCPLA,C.CDPRATCONSPL,C.CDFAPRATCPLA,'0','Z',NULL,0,0,'S') ) R ON R.CDPRODPRODUCAO IS NOT NULL AND R.IDTPREQPROD <> 'E' AND R.IDFINALIZACAO = 'F' INNER JOIN LISTA_SERVICO S ON S.NRLISTA = :P_NRLISTA AND S.CDFILIAL = C.CDFILICPLA AND S.CDSERVFILI = C.CDSERVCPLA WHERE C.NRCONSPLAN =:P_NRCONSPLAN AND ( E_AGREGADA(:P_CDCENTRAL,C.CDFILICPLA,C.CDSERVCPLA,C.CDTPPRATCPLA,C.DTCONSPLAN) > 0 ) AND C.IDTPREQPROD <> 'E' AND C.QTCOMPLPRODCPLA > 0 AND R.CDPRODUTO = C.CDPRODCPLA GROUP BY C.CDPRATCONSPL, C.CDFAPRATCPLA, C.CDFILICPLA, C.CDSERVCPLA, C.DTCONSPLAN, R.CDPRODPRODUCAO, R.CDPRODUTO, C.CDPRODCPLA, R.IDTPREQPROD, R.IDFINALIZACAO, S.CDGRUPSERV ) CONS GROUP BY CONS.DTCONSPLAN, CONS.CDPRODUTO, CONS.CDPRODPRODUCAO, CONS.IDTPREQPROD, CONS.CDGRUPSERV ) MP GROUP BY MP.DT, MP.CDPRODPRODUCAO, MP.CDPRODUTO, MP.QTCOMENSAIS, MP.IDTPREQPROD, MP.CDGRUPSERV UNION ALL SELECT CONS.DTCONSPLAN AS DT, NULL AS CDPRODPRODUCAO, CONS.CDPRODCPLA AS CDPRODUTO, SUM(CONS.QTCONSPLAN) AS QTPROD_PLA, SUM(CONS.QTCOMESTCPLA) AS QTCOMENSAIS, 'D' AS IDTPREQPROD, CONS.CDGRUPSERV FROM ( SELECT DISTINCT C.CDPRATCONSPL, C.CDFAPRATCPLA, C.CDFILICPLA, C.CDSERVCPLA, C.DTCONSPLAN, MAX(C.QTCOMESTCPLA) AS QTCOMESTCPLA, SUM(C.QTCONSPLAN) + ( SUM(NVL(C.QTCOMPLPRODCPLA,0) ) * MAX(C.QTCOMESTCPLA) ) AS QTCONSPLAN, C.CDPRODCPLA, C.IDCONSPLAN AS IDTPREQPROD, S.CDGRUPSERV FROM CONSPLAN C INNER JOIN LISTA_SERVICO S ON S.NRLISTA = :P_NRLISTA AND S.CDFILIAL = C.CDFILICPLA AND S.CDSERVFILI = C.CDSERVCPLA WHERE C.NRCONSPLAN =:P_NRCONSPLAN AND C.DTCONSPLAN BETWEEN :P_DTINI AND :P_DTFIN AND ( E_AGREGADA(:P_CDCENTRAL,C.CDFILICPLA,C.CDSERVCPLA,C.CDTPPRATCPLA,C.DTCONSPLAN) > 0 ) AND C.IDTPREQPROD <> 'E' AND C.IDCONSPLAN IN ( 'P', 'F' ) GROUP BY C.CDPRATCONSPL, C.CDFAPRATCPLA, C.CDFILICPLA, C.CDSERVCPLA, C.DTCONSPLAN, C.CDPRODCPLA, C.IDCONSPLAN, S.CDGRUPSERV ) CONS GROUP BY CONS.DTCONSPLAN, CONS.CDPRODCPLA, CONS.IDTPREQPROD, CONS.CDGRUPSERV ) C INNER JOIN PRODUTO P ON P.CDPRODUTO = C.CDPRODUTO LEFT JOIN PRODUTO PROD ON PROD.CDPRODUTO = C.CDPRODPRODUCAO INNER JOIN GRUPSERV S ON S.CDGRUPSERV = C.CDGRUPSERV WHERE P.CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN /*GROUP BY C.CDGRUPSERV, S.NMGRUPSERV, DT, C.CDPRODPRODUCAO, PROD.NMPRODUTO, P.CDARVPROD, C.IDTPREQPROD, P.NMPRODUTO, P.SGUNIDADE, P.NRORG*/ ORDER BY DT, DECODE(:P_ORDENA,'C',P.CDARVPROD,P.NMPRODUTO), C.CDGRUPSERV, C.QTCOMENSAIS DESC, DECODE(:P_ORDENA,'C',C.CDPRODPRODUCAO,PROD.NMPRODUTO)";
	    $params = array (
	        "P_CDCENTRAL"  => $row["CDCENTRAL"],
	        "P_NMCENTRAL"  => $row["NMCENTRAL"],
	        "P_DTINI"      => $row["DTINICIAL"],
	        "P_DTFIN"      => $row["DTFINAL"],
	        "P_NRLISTA"    => $row["NRLISTA"],
	        "P_NRCONSPLAN" => $row["NRCONSPLAN"],
	        "P_PRODINI"    => $row["CDARVPRODINI"],
	        "P_PRODFIN"    => $row["CDARVPRODFIN"],
	        "P_ORDENA"     => $row["ORDENA"]
	   );
	   
	    $data = $this->entityManager->getConnection()->fetchAll($sql, $params);
        $response->addDataset(new DataSet('relCustomizavel', array($data)));
	}
}